"""pyavreceiver errors."""


class AVReceiverInvalidArgumentError(Exception):
    """Invalid argument error."""


class AVReceiverIncompatibleDeviceError(Exception):
    """Invalid argument error."""
